<?php

namespace WebDevStudios;

defined( 'ABSPATH' ) || exit;

/**
 * Register the block with WordPress.
 * @since  1.7.0
 */
function register_block() {

	// Define our assets.
	$editor_script   = '../build/index.js';
	$editor_style    = '../build/index.css';
	$frontend_style  = '../build/style-index.css';
	$frontend_script = '../build/frontend.js';

	// Verify we have an editor script.
	if ( ! file_exists( plugin_dir_path( __FILE__ ) . $editor_script ) ) {
		wp_die( esc_html__( 'Whoops! You need to run `npm run build` for CPTUI-Extended first.', 'cptuiext' ) );
	}

	// Autoload dependencies and version.
	$asset_file = require plugin_dir_path( __FILE__ ) . '../build/index.asset.php';

	// Register editor script.
	wp_register_script(
		'cptui-extended-editor-script',
		plugins_url( $editor_script, __FILE__ ),
		$asset_file['dependencies'],
		$asset_file['version'],
		true
	);

	$post_types_to_list = format_post_types_for_block_dropdown();
	wp_add_inline_script(
		'cptui-extended-editor-script',
		'const cptuix_post_types = ' . json_encode( [
			'post_types' => $post_types_to_list
			] ),
		'after'
	);
	$productsPlugins = [
		'woocommerce' => class_exists( 'WooCommerce' ),
		'easydigitaldownloads' => class_exists( 'Easy_Digital_Downloads' ),
	];
	wp_add_inline_script(
		'cptui-extended-editor-script',
		'const hasProductPlugins = ' . json_encode( $productsPlugins ),
		'after'
	);

	// Register editor style.
	if ( file_exists( plugin_dir_path( __FILE__ ) . $editor_style ) ) {
		wp_register_style(
			'cptui-extended-editor-style',
			plugins_url( $editor_style, __FILE__ ),
			[ 'wp-edit-blocks' ],
			filemtime( plugin_dir_path( __FILE__ ) . $editor_style )
		);
	}

	// Register frontend style.
	if ( file_exists( plugin_dir_path( __FILE__ ) . $frontend_style ) ) {
		wp_register_style(
			'cptui-extended-style',
			plugins_url( $frontend_style, __FILE__ ),
			[],
			filemtime( plugin_dir_path( __FILE__ ) . $frontend_style )
		);
	}

	// Register block with WordPress.
	register_block_type( 'webdevstudios/cptui-extended', [
		'editor_script'   => 'cptui-extended-editor-script',
		'editor_style'    => 'cptui-extended-editor-style',
		'style'           => 'cptui-extended-style',
		'render_callback' => __NAMESPACE__ . '\construct_extended_shortcode',
		'attributes' => [
			'cptui_posttype'     => [ 'type' => 'string' ],
			'cptui_shortcode_id' => [ 'type' => 'integer' ],
			'cptui_shortcode'    => [ 'type' => 'string', 'default' => 'default_shortcode' ],
			'title'              => [ 'type' => 'string' ],
			'featured_image'     => [ 'type' => 'boolean', 'default' => true ],
			'excerpt'            => [ 'type' => 'boolean', 'default' => false ],
			'order_by'           => [ 'type' => 'string', 'default' => 'date' ],
			'order'              => [ 'type' => 'string', 'default' => 'DESC' ],
			'amount'             => [ 'type' => 'string' ],
			'list_type'          => [ 'type' => 'string', 'default' => 'ul' ],
			'post_id'            => [ 'type' => 'string' ],
			'attached_post'      => [ 'type' => 'string' ],
			'taxonomyItems'      => [ 'type' => 'string' ],
			'height'             => [ 'type' => 'string' ],
			'show_title'         => [ 'type' => 'boolean', 'default' => true ],
			'show_bullets'       => [ 'type' => 'boolean', 'default' => false ],
			'autoplay'           => [ 'type' => 'boolean', 'default' => true ],
			'card_layout'        => [ 'type' => 'string' ],
			'show_categories'    => [ 'type' => 'boolean', 'default' => false ],
			'featured_plus_type' => [ 'type' => 'string', 'default' => 'top' ],
			'show_read_more'     => [ 'type' => 'boolean', 'default' => false ],
			'show_post_date'     => [ 'type' => 'boolean', 'default' => false ],
		]
	] );

	// Register frontend script.
	if ( file_exists( plugin_dir_path( __FILE__ ) . $frontend_script ) ) {
		wp_enqueue_script(
			'cptui-extended-frontend-script',
			plugins_url( $frontend_script, __FILE__ ),
			$asset_file['dependencies'],
			$asset_file['version'],
			true
		);
	}
}
add_action( 'init', __NAMESPACE__ . '\register_block' );

/**
 * Pieces together a wrapper and our shortcode output based on passed block attributes.
 *
 * @since 1.7.0
 *
 * @param array $attributes Array of attributes for the block being rendered.
 * @return false|string
 */
function construct_extended_shortcode( $attributes ) {

	ob_start();
	echo ( ! empty( $attributes['className'] ) ) ? "<div class='{$attributes['className']}'>" : '<div>';
	$shortcode = '[cptui ';

	$attribute_pieces = [];
	$attribute_template = '%s="%s"';
	unset( $attributes['className'] );
	$is_woo = ( 'woo_product_shortcode' === $attributes[ 'cptui_shortcode' ] );
	$is_edd = ( 'edd_download_shortcode' === $attributes['cptui_shortcode'] );
	foreach ( $attributes as $attribute_name => $attribute_value ) {
		if ( 'taxonomyItems' === $attribute_name ) {
			$taxonomies_and_terms = json_decode( $attribute_value );
			foreach ( $taxonomies_and_terms as $taxonomy_and_terms ) {
				foreach ( $taxonomy_and_terms as $taxonomy => $terms ) {
					$attribute_pieces[] = sprintf(
						$attribute_template,
						"taxonomy-{$taxonomy}",
						implode( ',', $terms )
					);
				}
			}
		} else {
			if ( $is_woo && 'attached_post' === $attribute_name ) {
				$attribute_pieces[] = sprintf(
					$attribute_template,
					'attached_product',
					$attribute_value
				);
			} else if ( $is_edd && 'attached_post' === $attribute_name ) {
				$attribute_pieces[] = sprintf(
					$attribute_template,
					'attached_download',
					$attribute_value
				);
			} else {
				$attribute_pieces[] = sprintf(
					$attribute_template,
					$attribute_name,
					$attribute_value
				);
			}
		}

	}
	$shortcode .= implode( ' ', $attribute_pieces );
	$shortcode .= '][/cptui]';

	echo do_shortcode( $shortcode );
	echo '</div>';
	return ob_get_clean();
}

/**
 * Converts saved attributes to version that meets CPTUI Extended shortcode values.
 *
 * @since 1.7.0
 * @param object $attributes Object of attributes parsed from our shortcode.
 *
 * @return object
 */
function block_convert_attributes( $attributes ) {
	if ( ! empty( $attributes->featured_image ) ) {
		$attributes->featured_image = ( '1' === $attributes->featured_image || 'on' === $attributes->featured_image ) ? 'on' : 'off';

		if ( in_array(
			$attributes->cptui_shortcode,
			array( 'grid_shortcode', 'grid_with_overlay_shortcode', 'post_cards_shortcode' )
		)
		) {
			$attributes->show_thumbnail = ( '1' === $attributes->featured_image || 'on' === $attributes->featured_image ) ? 'on' : 'off';
		}
	}

	if ( ! empty( $attributes->excerpt ) ) {
		$attributes->excerpt = ( '1' === $attributes->excerpt || 'on' === $attributes->excerpt ) ? 'on' : 'off';

		if ( in_array(
			$attributes->cptui_shortcode,
			array( 'grid_shortcode', 'grid_with_overlay_shortcode', 'post_cards_shortcode' )
		)
		) {
			$attributes->show_excerpt = ( '1' === $attributes->excerpt || 'on' === $attributes->excerpt ) ? 'on' : 'off';
		}
	}

	if ( ! empty( $attributes->show_title ) ) {
		$attributes->show_title = ( '1' === $attributes->show_title || 'on' === $attributes->show_title ) ? 'on' : 'off';
	}

	if ( ! empty( $attributes->show_categories ) ) {
		$attributes->show_categories = ( '1' === $attributes->show_categories || 'on' === $attributes->show_categories ) ? 'on' : 'off';
	}

	if ( ! empty( $attributes->show_title ) ) {
		$attributes->show_title = ( '1' === $attributes->show_title || 'on' === $attributes->show_title ) ? 'on' : 'off';
	}

	if ( ! empty( $attributes->show_bullets ) ) {
		$attributes->show_bullets = ( '1' === $attributes->show_bullets || 'on' === $attributes->show_bullets ) ? 'on' : 'off';
	}

	if ( ! empty( $attributes->autoplay ) ) {
		$attributes->autoplay = ( '1' === $attributes->autoplay || 'on' === $attributes->autoplay ) ? 'on' : 'off';
	}

	if ( ! empty( $attributes->show_read_more ) ) {
		$attributes->show_read_more = ( '1' === $attributes->show_read_more || 'on' === $attributes->show_read_more ) ? 'on' : 'off';
	}

	if ( ! empty( $attributes->show_post_date ) ) {
		$attributes->show_post_date = ( '1' === $attributes->show_post_date || 'on' === $attributes->show_post_date ) ? 'on' : 'off';
	}

	return $attributes;
}
add_filter( 'cptui_shortcode_atts', __NAMESPACE__ . '\block_convert_attributes' );

/**
 * Retrieve public post types and turn into a formatted array for Select Component.
 *
 * @since 1.7.0
 *
 * @return array
 */
function format_post_types_for_block_dropdown() {
	$post_types           = get_post_types( array( 'public' => true ), 'objects' );
	$post_types_formatted = array();

	$reserved = array(
		'attachment',
		'revision',
		'nav_menu_item',
		'action',
		'order',
		'theme',
	);

	foreach ( $reserved as $post_type ) {
		unset( $post_types[ $post_type ] );
	}

	foreach( $post_types as $post_type ) {
		$post_types_formatted[] = array( 'label' => $post_type->label, 'value' => $post_type->name );
	}
	return $post_types_formatted;
}

/**
 * Load our main CSS in the admin for render preview purposes.
 *
 * @since 1.7.0
 */
function admin_load_scripts_styles() {
	wp_enqueue_style( 'cptui_default_css' );
	wp_enqueue_style( 'cptui_slick_slider_css' );
	wp_enqueue_style( 'cptui_slick_slider_theme' );
	wp_enqueue_script( 'cptui_slick_slider_js' );
	wp_enqueue_script( 'cptui_slider_js' );
}
add_action( 'admin_enqueue_scripts', __NAMESPACE__ . '\admin_load_scripts_styles', 11 );

/**
 * Add REST API support to EDD "Download" post type for post search support.
 *
 * @since 1.10.0
 * @param array $download_args Download post type arguments.
 *
 * @return array
 */
function add_download_rest_api_support( $download_args ) {
	$download_args['show_in_rest'] = true;

	return $download_args;
}
add_filter( 'edd_download_post_type_args', __NAMESPACE__ . '\add_download_rest_api_support' );
